/*
Capture patch <http://quakecapture.sourceforge.net/>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "IWriter.h"

#ifdef _WIN32
// This is a Win32 only implementation of ICapture for
// audio/video capture of AVI using a specified codec.
//  --Anthony.

#include <windows.h>
#include <vfw.h>

namespace Capture {

    class AviWriter : public IWriter
    {
    public:
        AviWriter(char* codec00, float fps, int audio_hz);
        virtual ~AviWriter();

        virtual bool Open(char* filename);
        virtual bool WriteVideo(int width, int height, unsigned char* rgb_pixel_buffer);
        virtual bool WriteAudio(int samples, unsigned char* stereo_16bit_sample_buffer);

    protected:
        typedef void (CALLBACK* LPFNAVIFILEINIT)(void);
        typedef HRESULT (CALLBACK* LPFNAVIFILEOPEN)(PAVIFILE*, LPCTSTR, UINT, LPCLSID);
        typedef HRESULT (CALLBACK* LPFNAVIFILECREATESTREAM)(PAVIFILE, PAVISTREAM*, AVISTREAMINFO*);
        typedef HRESULT (CALLBACK* LPFNAVIMAKECOMPRESSEDSTREAM)(PAVISTREAM*, PAVISTREAM, AVICOMPRESSOPTIONS*, CLSID*);
        typedef HRESULT (CALLBACK* LPFNAVISTREAMSETFORMAT)(PAVISTREAM, LONG, LPVOID, LONG);
        typedef HRESULT (CALLBACK* LPFNAVISTREAMWRITE)(PAVISTREAM, LONG, LONG, LPVOID, LONG, DWORD, LONG*, LONG*);
        typedef ULONG (CALLBACK* LPFNAVISTREAMRELEASE)(PAVISTREAM);
        typedef ULONG (CALLBACK* LPFNAVIFILERELEASE)(PAVIFILE);
        typedef void (CALLBACK* LPFNAVIFILEEXIT)(void);

        HINSTANCE m_avidll_handle;              
        LPFNAVIFILEINIT m_lpfnAVIFileInit;
        LPFNAVIFILEOPEN m_lpfnAVIFileOpen;
        LPFNAVIFILECREATESTREAM m_lpfnAVIFileCreateStream;
        LPFNAVIMAKECOMPRESSEDSTREAM m_lpfnAVIMakeCompressedStream;
        LPFNAVISTREAMSETFORMAT m_lpfnAVIStreamSetFormat;
        LPFNAVISTREAMWRITE m_lpfnAVIStreamWrite;
        LPFNAVISTREAMRELEASE m_lpfnAVIStreamRelease;
        LPFNAVIFILERELEASE m_lpfnAVIFileRelease;
        LPFNAVIFILEEXIT m_lpfnAVIFileExit;

        PAVIFILE m_file;
        PAVISTREAM m_uncompressed_video_stream;
        PAVISTREAM m_compressed_video_stream;
        PAVISTREAM VideoStream() const;
        PAVISTREAM m_uncompressed_audio_stream;
        PAVISTREAM AudioStream() const;

        unsigned long m_fourcc;
        int m_video_frame_counter;
        int m_video_frame_size;

        int m_audio_frame_counter;
        WAVEFORMATEX m_wave_format;
    };

}

#else
// NULL implementation for non-Win32 platforms

namespace Capture {

    void SetError(char*);

    class AviWriter : public IWriter
    {
    public:
        AviWriter(char* codec00, float fps, int audio_hz) : IWriter(codec00, fps, audio_hz)
        {
            SetError("Capture: avi mode not available on non-Win32 platforms\n");
        };
        virtual ~AviWriter() { };

        virtual bool Open(char*)                            { return false; };
        virtual bool WriteVideo(int, int, unsigned char*)   { return false; };
        virtual bool WriteAudio(int, unsigned char*)        { return false; };
    };

};

#endif