/*
Capture patch <http://quakecapture.sourceforge.net/>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "bridge.h"

#include <stdio.h>
#include <string.h>
#include "AviWriter.h"
#include "ScreenShotWriter.h"


Capture::IWriter* g_writer00 = NULL;
char g_capture_error[256];

namespace Capture {

    void SetError_s(char* format, char* string)
    {
        sprintf(g_capture_error, format, string);
    }

    void SetError(char* message)
    {
        strcpy(g_capture_error, message);
    }

    IWriter* MakeWriter00(char* mode, char* codec00, float fps, int audio_hz)
    {
        if (strcmp(mode, "avi") == 0) {
            return new AviWriter(codec00, fps, audio_hz);
        }

        if (strcmp(mode, "screenshot") == 0) {
            return new ScreenShotWriter(codec00, fps, audio_hz);
        }

        // default
        SetError_s("Capture: mode '%s' not recognised.\n", mode);
        return NULL;
    }

}

int Capture_Open(char* mode, char* basename, char* codec00, float fps, int audio_hz)
{
    char filename[256];
    char* i = filename - 1;
    
    if (*codec00 == '0') codec00 = 0;

    g_writer00 = Capture::MakeWriter00(mode, codec00, fps, audio_hz);
    if (!Capture_IsRecording()) return 0;

    strcpy(filename, basename);
    while (*++i) { if (*i == '/') *i = '\\'; } // replace slashes to end of string
    while ((*--i != '.') && (i > filename));   // work back to find an extension
    if (i > filename) *i = '\0';               // remove it if you find it
    
    if (g_writer00->Open(filename)) return 1;

    Capture_Close();
    return 0;
}

int Capture_WriteVideo(int width, int height, unsigned char* rgb_pixel_buffer)
{
    return Capture_IsRecording() &&
        g_writer00->WriteVideo(width, height, rgb_pixel_buffer);
}

int Capture_WriteAudio(int samples, unsigned char* stereo_16bit_sample_buffer)
{
    return Capture_IsRecording() &&
        g_writer00->WriteAudio(samples, stereo_16bit_sample_buffer);
}

void Capture_Close()
{
    delete g_writer00;
    g_writer00 = NULL;
}

int Capture_IsRecording()
{
    return (g_writer00 == NULL) ? 0 : 1;
}

char* Capture_Error()
{
    return g_capture_error;
}
