/*
Capture patch <http://quakecapture.sourceforge.net/>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

// Bridge to the C++ capture writers

#ifdef __cplusplus
extern "C" {
#endif

#ifndef __Q_CAPTUREBRIDGE__
#define __Q_CAPTUREBRIDGE__

/*  Capture_Open: starts a capture.
 *  Returns non-zero for success, zero for failure.
 *
 *  mode:       A string indicating the kind of capture.
 *              See implementation for allowable values.
 *
 *  filename:   A path relative to the current directory.
 *              (Not necessarily the Quake directory.)
 *              Can use Unix/Quake or Windows slash separators.
 *`             Ends with a base filename that can be used
 *              as the capture mode desires; typically any
 *              existing extension will be stripped, and replaced
 *              with one appropriate for the type of output.
 *
 *  codec:      A string indicating an optional codec. Again
 *              the interpretation is up to the capturer.
 *              "0" indicates no codec should be used.
 *
 *  fps:        Number of video frames per second
 *
 *  audio_hz:   Number of audio samples per second
 */
int Capture_Open(char* mode, char* filename, char* codec00, float fps, int audio_hz);

/*  Capture_WriteVideo: write a frame of video to an open capture.
 *  Returns non-zero for success, zero for failure.
 *
 *  width,      
 *   height:    Dimensions of the supplied video frame bitmap .
 *  
 *  buffer:     Video frame bitmap, three bytes per pixel.
 */
int Capture_WriteVideo(int width, int height, unsigned char* rgb_pixel_buffer);

/*  Capture_WriteAudio: write audio samples to an open capture.
 *  Returns non-zero for success, zero for failure.
 *
 *  samples:    Number of supplied audio samples
 *
 *  buffer:     Audio samples in the Quake (stereo, 16bit per sample) format
 */
int Capture_WriteAudio(int samples, unsigned char* stereo_16bit_sample_buffer);

/*  Capture_IsRecording: is there a current open capture?
 *  Returns non-zero if a capture is in progress, zero if not.
 */
int Capture_IsRecording(void);

/*  Capture_Close: close (complete) any open capture.
 *  Always succeeds.
 */
void Capture_Close(void);

/*  Capture_Error: reports an explicitly set error message.
 *	Returns the message.
 */
char* Capture_Error(void);

#endif

#ifdef __cplusplus
}
#endif