/*
Capture patch <http://quakecapture.sourceforge.net/>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

// This file contains the Quake1-specific parts of the capture patch

#include "helpers.h"
#include "../quakedef.h"
#include "../sys.h"

extern cvar_t host_framerate;
extern qboolean scr_drawloading;

void CaptureHelper_Printf_f(char* format, float float_arg)
{
    Con_Printf(format, float_arg);
}

void CaptureHelper_Printf_s(char* format, char* string_arg)
{
    Con_Printf(format, string_arg);
}

void CaptureHelper_Print(char* string)
{
    Con_Printf("%s", string);
}

float CaptureHelper_VideoFps(void)
{
    if (host_framerate.value) return 1.0f / host_framerate.value;
    else return 0.0;
}

void CaptureHelper_SetVideoFps(float fps)
{
    if (fps == 0.0) {
        Cvar_SetValue("host_framerate", 0.0);
        return;
    }
    Cvar_SetValue("host_framerate", 1.0f / fps);
    if (Cvar_VariableValue("_snd_mixahead") < 1.0 / fps) {
        CaptureHelper_Print("Warning: you may want to increase _snd_mixahead!\n");
    }
    if (fps > 72.0) {
        CaptureHelper_Print("Warning: fps set above the usual hard coded maximum of 72. ");
        CaptureHelper_Print("You may need to patch Host_FilterTime in host.c source code!\n");
    }
}

int CaptureHelper_AudioHz(void)
{
    return shm->speed;
}

qboolean CaptureHelper_ShouldSuppressCapture(void)
{
    return 
        // don't output whilst console is down or 'loading' is displayed
        (scr_con_current > 0) ||
        (scr_drawloading);
}

qboolean CaptureHelper_CopyVideo(unsigned char* buffer)
{
#ifdef GLQUAKE
    unsigned char temp;
    int i;
    int size = glwidth * glheight * 3;

	glReadPixels (glx, gly, glwidth, glheight, GL_RGB, GL_UNSIGNED_BYTE, buffer);
	for (i = 0; i < size; i += 3) {
		temp = buffer[i];
		buffer[i] = buffer[i+2];
		buffer[i+2] = temp;
	}
    return true;
#else
    CaptureHelper_Print("Capture not implemented for software rendering.\n");
    return false;
#endif
}

qboolean CaptureHelper_PlayDemo_f(void)
{
#ifdef GLQUAKE
    CL_PlayDemo_f();
    return cls.demoplayback;
#else
    CaptureHelper_Print("Capture not implemented for software rendering.\n");
    return false;
#endif
}

void CaptureHelper_SetBatchCapture(qboolean batch)
{
    cls.capturedemo = batch;
}

qboolean CaptureHelper_BatchCapture(void)
{
    return cls.capturedemo;
}

int CaptureHelper_VideoWidth(void)
{
#ifdef GLQUAKE
    return glwidth;
#else
    return 1;
#endif
}

int CaptureHelper_VideoHeight(void)
{
#ifdef GLQUAKE
    return glheight;
#else
    return 1;
#endif
}

int CaptureHelper_SoundChannels(void)
{
    return shm->channels;
}

char* CaptureHelper_DefaultDemoExtension(void)
{
    return ".dem";
}

cvar_t capturehelper_cvars[3];
int capturehelper_num_cvars = 3;
void CaptureHelper_InitVariable(char* variablename, char* defaultvalue)
{
    cvar_t* new_cvar = capturehelper_cvars + --capturehelper_num_cvars;
    if (capturehelper_num_cvars < 0) {
        Sys_Error("CaptureHelper_InitVariable: logic error: ran out of capturehelper_cvars");
    }
    new_cvar->name = variablename;
    new_cvar->string = defaultvalue;
    new_cvar->archive = true;
    Cvar_RegisterVariable(new_cvar);
}

void CaptureHelper_Shutdown(void)
{
	CL_Disconnect();
	Host_ShutdownServer(false);		
	Sys_Quit();
}
